#region Copyright Syncfusion Inc. 2001 -2004
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////// 
//  Copyright Syncfusion Inc. All rights reserved. Use of this code is subject to the terms of our 
//  license. A copy of the current license can be obtained at any time by e-mailing licensing@syncfusion.com. 
//  Re-distribution in any form is strictly prohibited. Any infringement will be prosecuted under applicable laws. 
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////// 
#endregion 
 
using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.IO;
using System.Xml;
using System.Reflection;
using System.Diagnostics;
using System.Resources;

namespace Syncfusion.Samples.GridPad
{
	/// <summary>
	///    Summary description for MainFrame.
	/// </summary>
	public class MainWindow : System.Windows.Forms.Form
	{
		/// <summary>
		///    Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components;
		protected MenuItem lastfiles;
		public ArrayList BufferOpenMenuItems = new ArrayList();
		public StatusBar      statusBar       = new StatusBar();
		public StatusBarPanel statusBarPanel  = new StatusBarPanel();
		StatusBarPanel statusBarPanel1 = new StatusBarPanel();
		StatusBarPanel statusBarPanel2 = new StatusBarPanel();
		protected ToolBar standardtoolbar;

		public MainWindow()
		{
			InitializeComponent();

			try
			{
				Type type = typeof(MainWindow);
				Stream stream = type.Module.Assembly.GetManifestResourceStream(ApplicationMain.manifestNamespace + "MainMenuItems.xml");
				StreamReader reader = new StreamReader(stream);
				XmlDocument mainmenu = new XmlDocument();
				mainmenu.Load(reader); 
				MenuItem[] _items = CreateMenu(typeof(BasicAction).Module.Assembly, ApplicationMain.menuNamespace, mainmenu.DocumentElement);
				ArrayList items = new ArrayList();
				foreach (MenuItem i in _items)
				{
					if (i != null)
					{
						i.OwnerDraw = true;
						items.Add(i);
					}
				}

				Menu = new MainMenu((MenuItem[]) items.ToArray(typeof(MenuItem)));

				StandardStatusBar(null, null);
				statusBarPanel.BorderStyle = StatusBarPanelBorderStyle.None;
				statusBarPanel.Width = 500;
				statusBarPanel.AutoSize = StatusBarPanelAutoSize.Spring;
				
				statusBarPanel1.BorderStyle = StatusBarPanelBorderStyle.Sunken;
				statusBarPanel1.Width = 50;
				statusBarPanel1.AutoSize = StatusBarPanelAutoSize.None;

				statusBarPanel2.BorderStyle = StatusBarPanelBorderStyle.Sunken;
				statusBarPanel2.Width = 50;                      
				statusBarPanel2.AutoSize = StatusBarPanelAutoSize.None;
				statusBar.ShowPanels = true;
				statusBar.Panels.Add(statusBarPanel);
				statusBar.Panels.Add(statusBarPanel1);
				statusBar.Panels.Add(statusBarPanel2);

				this.IsMdiContainer = true;
				Anchor = ~AnchorStyles.None;
				MdiChildActivate += new EventHandler(UpdateOpenStatus);
				MenuComplete += new EventHandler(StandardStatusBar);
				
				// set the old form start position
				this.Size = new System.Drawing.Size(640, 480);
				UpdateOpenStatus(null, null);
				UpdateLastFiles();

				this.Controls.Add(statusBar);

				standardtoolbar      = new StandardToolBar(this);
				standardtoolbar.Dock = DockStyle.Top;
                
				this.Controls.Add(standardtoolbar);
			}  
			catch
			{
			}  
		}

		/// <summary>
		///    Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		/// <summary>
		///    Required method for Designer support - do not modify
		///    the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			this.Size = new System.Drawing.Size(300,300);
			this.Text = "GridPad";
			try
			{
				this.Icon = (Icon) ApplicationMain.IconResources.GetObject("GridPad");
			}
			catch
			{
			}

		}

		void UpdateOpenStatus(object sender, EventArgs e)
		{
			foreach (SmartMenuItem item in BufferOpenMenuItems) 
				item.Enabled = item.hasHandler; 
		}
		
		void StandardStatusBar(object sender, EventArgs e)
		{
			statusBarPanel.Text = "Ready";
		}

		public void UpdateLastFiles()
		{
			if (lastfiles != null) 
			{
				lastfiles.MenuItems.Clear();
				MenuItem tmp = new MenuItem("last files");
				tmp.Enabled  = false;
				lastfiles.MenuItems.Add(tmp);
			}
		}

		void SelectMenuItem(object sender, EventArgs e)
		{
			if (sender.GetType().Equals(typeof (SmartMenuItem))) 
			{
				SmartMenuItem item = (SmartMenuItem)sender;
				if (item.Description.Length > 0)
					statusBarPanel.Text = item.Description;
				else
					statusBarPanel.Text = "";
			}
		}
		
		public MenuItem[] CreateMenu(Assembly asm, string eventnamespace, XmlElement xmlelement)
		{
			XmlNodeList nodes = xmlelement.ChildNodes;
			
			SmartMenuItem[] item = new SmartMenuItem[nodes.Count];
			
			for (int i = 0; i < nodes.Count; ++i) 
			{
				XmlElement el = (XmlElement)nodes[i];
				EventHandler handler = null;
				
				XmlAttribute attribute = el.Attributes["attribute"];
				
				// set eventhandler
				if (el.HasAttribute("event")) 
				{
					BasicAction ia = null;
					string objname = eventnamespace + el.Attributes["event"].InnerText;
					ia = (BasicAction)asm.CreateInstance(objname);

					if (ia == null) 
					{
						Trace.WriteLine("Can't create MenuAction " + el.Attributes["event"].InnerText);
					} 
					else 
					{
						ia.MainWindow = this;
						handler = new EventHandler(ia.InvokeAction);
					}
				}
				
				// create MenuItem
				XmlAttribute nameItem = el.HasAttribute("name") ? el.Attributes["name"] : null;
				XmlAttribute descItem = el.HasAttribute("description") ? el.Attributes["description"] : null;
				string s1 = nameItem == null ? "" : el.Attributes["name"].InnerText;
				string s2 = descItem == null ? "" : el.Attributes["description"].InnerText;
				item[i] = new SmartMenuItem(s1, handler, s2);
				item[i].Select += new EventHandler(SelectMenuItem);
				// quick hack so that menus are grayed
				if (el.HasAttribute("event")) 
					BufferOpenMenuItems.Add(item[i]);
				
				// read attributes
				if (attribute != null)
					switch (attribute.InnerText.ToUpper()) 
					{
						case "LASTFILESMENU":
							lastfiles = item[i];
							break;
						case "MDILIST":
							item[i].MdiList = true;
							break;
					}
				
				// set menuitem icon
				XmlAttribute icon     = el.HasAttribute("icon") ? el.Attributes["icon"] : null;
				if (icon != null) 
				{
					try
					{
						object obj = ApplicationMain.IconResources.GetObject(icon.InnerText);
						item[i].Icon = (Bitmap)obj;
					}
					catch (Exception ex)
					{
						Trace.WriteLine("Icon " + icon.InnerText + " could not be found: " + ex.ToString());
					}
				}
				
				// set menuitem shortcut
				XmlAttribute shortcut1 = el.HasAttribute("shortcut") ? el.Attributes["shortcut"] : null;
				if (shortcut1 != null) 
				{
					try 
					{ 
						Type sType = Shortcut.F1.GetType();
						item[i].Shortcut = (Shortcut) sType.InvokeMember(shortcut1.InnerText, BindingFlags.GetField, null, Shortcut.F1, new object[0]);
					} 
					catch (Exception) 
					{
						item[i].Shortcut = Shortcut.None;
						item[i].ShortcutText = shortcut1.InnerText;
					}
				} 
				
				// add submenus
				MenuItem[] submenus = CreateMenu(asm, eventnamespace, el);
				foreach (MenuItem menuitem in submenus)
					item[i].MenuItems.Add(menuitem);
			}

			return item;
		}

	}
}
