#region Copyright Syncfusion Inc. 2001 -2004
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////// 
//  Copyright Syncfusion Inc. All rights reserved. Use of this code is subject to the terms of our 
//  license. A copy of the current license can be obtained at any time by e-mailing licensing@syncfusion.com. 
//  Re-distribution in any form is strictly prohibited. Any infringement will be prosecuted under applicable laws. 
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////// 
#endregion 
 
using System;
using System.Collections;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Text;
using System.IO;
using System.Reflection;
using System.Windows.Forms;
using System.Xml;

namespace Syncfusion.Samples.GridPad 
{
    /// <summary>
    ///    Summary description for SmartMenuItem.
    /// </summary>
    public class SmartMenuItem : MenuItem
    {
        private void InitializeComponent ()
        {
        }
    
        static int TEXTSTART = 20;
        public string ShortcutText = "";
        public string Description = null;
        static Font   myfont = new Font("Tahoma", 8);
        public bool hasHandler = false;
		
        Bitmap _icon   = null;
		
        public Bitmap Icon 
        {
            get 
            {
                return _icon;
            }
            set 
            {
                _icon = value;
                _icon.MakeTransparent(_icon.GetPixel(0, 0));
            }
        }
		
        protected override void OnMeasureItem(MeasureItemEventArgs e)
        {
            base.OnMeasureItem(e);
            bool isMainMenu    = Object.ReferenceEquals(this.Parent, this.GetMainMenu());
            
            if (isMainMenu)
            {
                e.ItemHeight = SystemInformation.MenuHeight+4;
                if (Text.Equals("-"))
                    e.ItemWidth = 4;
                else
                    e.ItemWidth = (int) e.Graphics.MeasureString(Text, myfont, 120, stringformat).Width;
            }
            else
            {
                if (Text.Equals("-"))
                {
                    e.ItemHeight = 4;
                    e.ItemWidth = 4;
                }
                else
                {
                    if (Shortcut != Shortcut.None) 
                    {
                        string text = "";
                        int    key  = (int)Shortcut;
                        int    ch   = key & 0xFF;
                        if (((int)Keys.Control & key) > 0)
                            text += "Ctrl+";
                        if (((int)Keys.Shift & key) > 0)
                            text += "Shift+";
                        if (((int)Keys.Alt & key) > 0)
                            text += "Alt+";
			        	
                        if (ch >= (int)Shortcut.F1 && ch <= (int)Shortcut.F12)
                            text += "F" + (ch - (int)Shortcut.F1 + 1);
                        else
                            text += (char)ch;
                        if (text.Length > 0)
                            ShortcutText = "   " + text;
                    }
                    e.ItemHeight = SystemInformation.MenuHeight;
                    e.ItemWidth = Math.Max(120, (int)(e.Graphics.MeasureString(Text + ShortcutText, myfont).Width) + 50);
                }
            }
        }
		
        bool         selected = false;
        StringFormat stringformat = new StringFormat();
		
        void DrawItemText(Graphics g, string text, Color color, int x, int y)
        {
            if (Enabled) 
            {
                // normal draw
                g.DrawString(text, myfont, new SolidBrush(color), x, y, stringformat);
            } 
            else 
            {
                // disabled menuitem draw
                if (!selected)
                    g.DrawString(text, myfont, SystemBrushes.ControlLightLight, x + 1, y + 1, stringformat);
                g.DrawString(text, myfont, new SolidBrush(SystemColors.GrayText), x, y, stringformat);
            }
        }
		
        protected override void OnSelect(EventArgs e)
        {
            base.OnSelect(e);
        }
		
        protected override void OnDrawItem(DrawItemEventArgs e)
        {
            Graphics  g        = e.Graphics;
            Rectangle drawrect = e.Bounds;
            Rectangle insiderect = drawrect;
            bool isMainMenu    = Object.ReferenceEquals(this.Parent, this.GetMainMenu());

			if (isMainMenu)
			{
				drawrect.Height--;
				insiderect.Height--;
			}

			insiderect.Inflate(-2, 0);
			insiderect.Height--;

            myfont             = e.Font;
			
            selected                  = (e.State & DrawItemState.Selected) > 0;
            stringformat.HotkeyPrefix = HotkeyPrefix.Show; //((e.State & DrawItemState.NoAccelerator) > 0) ? HotkeyPrefix.Hide : HotkeyPrefix.Show;

			
            base.OnDrawItem(e);

            if (Text == "-")
            {
                Brush brush = new SolidBrush(SystemColors.Menu);
                Pen pen = new Pen(Color.FromArgb(75,SystemColors.ControlDarkDark));
                pen.Width = 1.0f;
                try
                {
                    g.FillRectangle(brush,e.Bounds);
                    g.DrawLine(pen, new Point(insiderect.X + 1, insiderect.Y + (insiderect.Height/2)),
                        new Point((insiderect.X + insiderect.Width) - 2, insiderect.Y + (insiderect.Height/2))
                        );
                }  
                finally
                {
                    brush.Dispose();
                }  
                return;
            }


            if (!isMainMenu)
            {
                if (Icon != null) 
                {
                    Brush brush = new SolidBrush(SystemColors.Menu);
                    g.FillRectangle(brush, drawrect);
                    if (selected)
                        g.DrawImage(Icon, insiderect.Left + 1, insiderect.Top + 1);
                    else
                        g.DrawImage(Icon, insiderect.Left + 2, insiderect.Top + 2);
                    drawrect.X += Icon.Width + 4;
                    brush.Dispose();
                }
            }
			
            if (selected)
            {
		        Brush brush2 = new SolidBrush(SystemColors.Menu);
		        Brush brush3 = new SolidBrush(Color.FromArgb(70,SystemColors.ActiveCaption));
		        Brush brush4 = new SolidBrush(Color.FromArgb(70,SystemColors.Highlight));
		        Pen pen = new Pen(SystemColors.WindowFrame);
		        g.FillRectangle(brush2, drawrect);
		        g.FillRectangle(brush3, insiderect);
		        g.FillRectangle(brush4, insiderect);
		        g.DrawRectangle(pen, insiderect);
		        pen.Dispose();
		        brush4.Dispose();
		        brush3.Dispose();
		        brush2.Dispose();

				if (isMainMenu)
				{
					DrawItemText(g, Text, SystemColors.HighlightText, insiderect.Left + 4, insiderect.Top + 3);
				}
				else
					DrawItemText(g, Text, SystemColors.HighlightText, insiderect.Left + TEXTSTART, insiderect.Top + 3);
			
			}
            else
            {
                Brush   brush = new SolidBrush(!isMainMenu ? SystemColors.Control : SystemColors.Control);
                g.FillRectangle(brush, drawrect);
                brush.Dispose();
			
				if (isMainMenu)
				{
					DrawItemText(g, Text, e.ForeColor, insiderect.Left + 4, insiderect.Top + 3);
				}
				else
					DrawItemText(g, Text, e.ForeColor, insiderect.Left + TEXTSTART, insiderect.Top + 3);
			}
			
            DrawItemText(g, ShortcutText, e.ForeColor, insiderect.Right - 55, insiderect.Top + 2);
        }
		
        public SmartMenuItem()
        {}
		
        public SmartMenuItem(string name, EventHandler handler, string description) : base(name, handler)
        {
            this.Description = description;
            OwnerDraw        = true;//name == "-";
            stringformat.HotkeyPrefix = HotkeyPrefix.Show;
            hasHandler = (handler != null);
        }
		
        public override MenuItem CloneMenu()
        {
            MenuItem mi = base.CloneMenu();
            mi.OwnerDraw = false;
            return mi; 
        }
    }
}
